package com.yonge.cooleshow.auth.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import com.yonge.toolset.mybatis.service.impl.BaseServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.stereotype.Service;

import com.yonge.cooleshow.auth.api.entity.SysUserDevice;
import com.yonge.cooleshow.auth.dal.dao.SysUserDeviceDao;
import com.yonge.cooleshow.auth.service.SysUserDeviceService;
import com.yonge.toolset.mybatis.dal.BaseDAO;

@Service
public class SysUserDeviceServiceImpl extends BaseServiceImpl<Long, SysUserDevice> implements SysUserDeviceService {
	
	@Autowired
	private SysUserDeviceDao sysUserDeviceDao;

	@Override
	public BaseDAO<Long, SysUserDevice> getDAO() {
		return sysUserDeviceDao;
	}

	@Override
	public List<SysUserDevice> queryByUserId(String clientId, Long userId) {
		return sysUserDeviceDao.queryByUserId(clientId, userId);
	}

	@Override
	public List<SysUserDevice> queryByDeviceNum(String clientId, String deviceNum) {
		return sysUserDeviceDao.queryByDeviceNum(clientId, deviceNum);
	}

	@Override
	public boolean bindDevice(String clientId, Long userId, String deviceNum) {

		if (StringUtils.equals(clientId, "STUDENT")) {
			// 检查学生是否绑定了多个设备号
			List<SysUserDevice> studentDeviceList = queryByUserId(clientId, userId);

			if (studentDeviceList == null) {
				studentDeviceList = new ArrayList<SysUserDevice>();
			}

			List<String> deviceList = studentDeviceList.stream().map(t -> t.getDeviceNum()).collect(Collectors.toList());

			if (!deviceList.contains(deviceNum)) {
				if (deviceList.size() >= 5) {
					throw new BadCredentialsException("当前账号绑定设备过多，请联系主教老师");
				}

				SysUserDevice sysUserDevice = new SysUserDevice();
				sysUserDevice.setUserId(userId);
				sysUserDevice.setDeviceNum(deviceNum);
				sysUserDevice.setBindTime(new Date());
				sysUserDevice.setClientId(clientId);
				sysUserDeviceDao.insert(sysUserDevice);
			}
		} else {
			// 查询设备号是否已存在
			List<SysUserDevice> sysUserDeviceList = sysUserDeviceDao.queryByDeviceNum(null, deviceNum);

			if (sysUserDeviceList == null) {
				sysUserDeviceList = new ArrayList<SysUserDevice>();
			}

			sysUserDeviceList = sysUserDeviceList.stream().filter(sud -> !StringUtils.equals(sud.getClientId(), "STUDENT")).collect(Collectors.toList());

			if (sysUserDeviceList != null && sysUserDeviceList.size() > 0) {
				if (sysUserDeviceList.stream().filter(sud -> sud.getUserId().equals(userId)).count() > 0) {

					return true;
				}

				throw new BadCredentialsException("当前设备已绑定账号，请更换设备");
			}

			SysUserDevice sysUserDevice = new SysUserDevice();
			sysUserDevice.setUserId(userId);
			sysUserDevice.setDeviceNum(deviceNum);
			sysUserDevice.setBindTime(new Date());
			sysUserDevice.setClientId(clientId);
			sysUserDeviceDao.insert(sysUserDevice);
		}

		return true;
	}

	@Override
	public boolean deleteWithLogic(Long id) {
		
		SysUserDevice sysUserDevice = sysUserDeviceDao.get(id);
		if(sysUserDevice != null){
			sysUserDevice.setDelFlag(false);
			
			sysUserDeviceDao.update(sysUserDevice);
		}
		
		return true;
	}
	
}