package com.yonge.cooleshow.teacher.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yonge.cooleshow.biz.dal.entity.SysConfig;
import com.yonge.cooleshow.biz.dal.service.SysConfigService;
import com.yonge.cooleshow.common.controller.BaseController;

/** 
 * 系统配置控制层
 */
@RestController
@Api(tags = "系统参数设置")
@RequestMapping(value = "sysConfig")
public class SysConfigController extends BaseController {

	@Autowired
	private SysConfigService sysConfigService;

	@ApiOperation(value = "参数列表")
	@GetMapping(value = "list")
	public Object configList(String group) {
		Map<String,Object> params = new HashMap<String, Object>();
		params.put("group", group);
		List<SysConfig> configs = sysConfigService.findAll(params);
		return succeed(configs);
	}

	@ApiOperation(value = "查询参数")
	@GetMapping(value = "get")
	public Object getConfig(Long id) {
		if (id == null || id <= 0)
			return failed("请检查输入的ID");
		return succeed(sysConfigService.get(id));
	}

	@ApiOperation(value = "查询参数")
	@GetMapping(value = "queryByParamName")
	public Object queryByParamName(String paramName) {
		if(StringUtils.isBlank(paramName)){
			return failed("参数不能为空");
		}
		return succeed(sysConfigService.findByParamName(paramName));
	}

	@ApiOperation(value = "查询参数")
	@GetMapping(value = "queryByParamNameList")
	public Object queryByParamNameList(String paramNames) {
		if(StringUtils.isBlank(paramNames)){
			return failed("参数不能为空");
		}
		List<String> paramNameList = Arrays.asList(paramNames.split(","));
		return succeed(sysConfigService.findByParamName(paramNameList));
	}
}
